@@ -1,20 +1,18 @@
 package CGI::Session;
-
-# $Id: Session.pm 353 2006-12-05 02:10:19Z markstos $
-
 use strict;
 use Carp;
 use CGI::Session::ErrorHandler;
 
 @CGI::Session::ISA      = qw( CGI::Session::ErrorHandler );
-$CGI::Session::VERSION  = '4.20';
+$CGI::Session::VERSION  = '4.48';
 $CGI::Session::NAME     = 'CGISESSID';
 $CGI::Session::IP_MATCH = 0;
 
-sub STATUS_NEW      () { 1 }        # denotes session that's just created
-sub STATUS_MODIFIED () { 2 }        # denotes session that needs synchronization
-sub STATUS_DELETED  () { 4 }        # denotes session that needs deletion
-sub STATUS_EXPIRED  () { 8 }        # denotes session that was expired.
+sub STATUS_UNSET    () { 1 << 0 } # denotes session that's resetted
+sub STATUS_NEW      () { 1 << 1 } # denotes session that's just created
+sub STATUS_MODIFIED () { 1 << 2 } # denotes session that needs synchronization
+sub STATUS_DELETED  () { 1 << 3 } # denotes session that needs deletion
+sub STATUS_EXPIRED  () { 1 << 4 } # denotes session that was expired.
 
 sub import {
     my ($class, @args) = @_;
@@ -22,7 +20,7 @@ sub import {
     return unless @args;
 
   ARG:
-    foreach my $arg (@args) {
+    for my $arg (@args) {
         if ($arg eq '-ip_match') {
             $CGI::Session::IP_MATCH = 1;
             last ARG;
@@ -56,11 +54,16 @@ sub new {
         #
         # Called as a class method as in CGI::Session->new()
         #
+
+        # Start fresh with error reporting. Errors in past objects shouldn't affect this one. 
+        $class->set_error('');
+
         $self = $class->load( @args );
         if (not defined $self) {
             return $class->set_error( "new(): failed: " . $class->errstr );
         }
     }
+
     my $dataref = $self->{_DATA};
     unless ($dataref->{_SESSION_ID}) {
         #
@@ -80,6 +83,7 @@ sub new {
         }
         $dataref->{_SESSION_ID} = $id;
         $dataref->{_SESSION_CTIME} = $dataref->{_SESSION_ATIME} = time();
+        $dataref->{_SESSION_REMOTE_ADDR} = $ENV{REMOTE_ADDR} || "";
         $self->_set_status( STATUS_NEW );
     }
     return $self;
@@ -128,7 +132,7 @@ sub _id_generator   {
     return $self->{_OBJECTS}->{id} = "CGI::Session::ID::" . $self->{_DSN}->{id};
 }
 
-sub _ip_matches {
+sub ip_matches {
   return ( $_[0]->{_DATA}->{_SESSION_REMOTE_ADDR} eq $ENV{REMOTE_ADDR} );
 }
 
@@ -163,15 +167,16 @@ sub query {
 
 sub name {
     my $self = shift;
+    my $name = shift;
     
     if (ref $self) {
-        unless ( @_ ) {
+        unless ( defined $name ) {
             return $self->{_NAME} || $CGI::Session::NAME;
         }
-        return $self->{_NAME} = $_[0];
+        return $self->{_NAME} = $name;
     }
     
-    $CGI::Session::NAME = $_[0] if @_;
+    $CGI::Session::NAME = $name if defined $name;
     return $CGI::Session::NAME;
 }
 
@@ -189,19 +194,19 @@ sub dump {
 sub _set_status {
     my $self    = shift;
     croak "_set_status(): usage error" unless @_;
-    $self->{_STATUS} |= $_ for @_;
+    $self->{_STATUS} |= $_[0];
 }
 
 
 sub _unset_status {
     my $self = shift;
     croak "_unset_status(): usage error" unless @_;
-    $self->{_STATUS} &= ~$_ for @_;
+    $self->{_STATUS} &= ~$_[0];
 }
 
 
 sub _reset_status {
-    $_[0]->{_STATUS} = 0;
+    $_[0]->{_STATUS} = STATUS_UNSET;
 }
 
 sub _test_status {
@@ -213,16 +218,18 @@ sub flush {
     my $self = shift;
 
     # Would it be better to die or err if something very basic is wrong here? 
-    # I'm trying to address the DESTORY related warning
+    # I'm trying to address the DESTROY related warning
     # from: http://rt.cpan.org/Ticket/Display.html?id=17541
     # return unless defined $self;
 
     return unless $self->id;            # <-- empty session
-    return if !defined($self->{_STATUS}) or $self->{_STATUS} == 0;    # <-- neither new, nor deleted nor modified
+    
+    # neither new, nor deleted nor modified
+    return if !defined($self->{_STATUS}) or $self->{_STATUS} == STATUS_UNSET;
 
     if ( $self->_test_status(STATUS_NEW) && $self->_test_status(STATUS_DELETED) ) {
         $self->{_DATA} = {};
-        return $self->_unset_status(STATUS_NEW, STATUS_DELETED);
+        return $self->_unset_status(STATUS_NEW | STATUS_DELETED);
     }
 
     my $driver      = $self->_driver();
@@ -236,14 +243,14 @@ sub flush {
         return $self->_unset_status(STATUS_DELETED);
     }
 
-    if ( $self->_test_status(STATUS_NEW) || $self->_test_status(STATUS_MODIFIED) ) {
+    if ( $self->_test_status(STATUS_NEW | STATUS_MODIFIED) ) {
         my $datastr = $serializer->freeze( $self->dataref );
         unless ( defined $datastr ) {
             return $self->set_error( "flush(): couldn't freeze data: " . $serializer->errstr );
         }
         defined( $driver->store($self->id, $datastr) ) or
             return $self->set_error( "flush(): couldn't store datastr: " . $driver->errstr);
-        $self->_unset_status(STATUS_NEW, STATUS_MODIFIED);
+        $self->_unset_status(STATUS_NEW | STATUS_MODIFIED);
     }
     return 1;
 }
@@ -431,10 +438,12 @@ sub find {
         return $class->set_error( "find(): couldn't create driver object. " . $pm->errstr );
     }
 
-    my $dont_update_atime = 0;
+    # Read-only isn't the perfect name here. In read-only mode, we skip the ip_match check,
+    # and don't update the atime. We *do* still delete expired sessions and session params.
+    my $read_only = 1;
     my $driver_coderef = sub {
         my ($sid) = @_;
-        my $session = $class->load( $dsn, $sid, $dsn_args, $dont_update_atime );
+        my $session = $class->load( $dsn, $sid, $dsn_args, $read_only );
         unless ( $session ) {
             return $class->set_error( "find(): couldn't load session '$sid'. " . $class->errstr );
         }
@@ -446,7 +455,7 @@ sub find {
     return 1;
 }
 
-# $Id: Session.pm 353 2006-12-05 02:10:19Z markstos $
+# $Id$
 
 =pod
 
@@ -458,89 +467,43 @@ CGI::Session - persistent session data in CGI applications
 
     # Object initialization:
     use CGI::Session;
-    $session = new CGI::Session();
+    $session = CGI::Session->new();
 
     $CGISESSID = $session->id();
 
-    # send proper HTTP header with cookies:
+    # Send proper HTTP header with cookies:
     print $session->header();
 
-    # storing data in the session
+    # Storing data in the session:
     $session->param('f_name', 'Sherzod');
     # or
     $session->param(-name=>'l_name', -value=>'Ruzmetov');
 
-    # flush the data from memory to the storage driver at least before your
-    # program finishes since auto-flushing can be unreliable
+    # Flush the data from memory to the storage driver at least before your
+    # program finishes since auto-flushing can be unreliable.
     $session->flush();
 
-    # retrieving data
+    # Retrieving data:
     my $f_name = $session->param('f_name');
     # or
     my $l_name = $session->param(-name=>'l_name');
 
-    # clearing a certain session parameter
+    # Clearing a certain session parameter:
     $session->clear(["l_name", "f_name"]);
 
-    # expire '_is_logged_in' flag after 10 idle minutes:
+    # Expire '_is_logged_in' flag after 10 idle minutes:
     $session->expire('is_logged_in', '+10m')
 
-    # expire the session itself after 1 idle hour
+    # Expire the session itself after 1 idle hour:
     $session->expire('+1h');
 
-    # delete the session for good
+    # Delete the session for good:
     $session->delete();
+    $session->flush(); # Recommended practice says use flush() after delete().
 
 =head1 DESCRIPTION
 
-CGI-Session is a Perl5 library that provides an easy, reliable and modular session management system across HTTP requests.
-Persistency is a key feature for such applications as shopping carts, login/authentication routines, and application that
-need to carry data across HTTP requests. CGI::Session does that and many more.
-
-=head1 TRANSLATIONS
-
-This document is also available in Japanese.
-
-=over 4
-
-=item o 
-
-Translation based on 4.14: http://digit.que.ne.jp/work/index.cgi?Perldoc/ja
-
-=item o
-
-Translation based on 3.11, including Cookbook and Tutorial: http://perldoc.jp/docs/modules/CGI-Session-3.11/
-
-=back
-
-=head1 TO LEARN MORE
-
-Current manual is optimized to be used as a quick reference. To learn more both about the philosophy and CGI::Session
-programming style, consider the following:
-
-=over 4
-
-=item *
-
-L<CGI::Session::Tutorial|CGI::Session::Tutorial> - extended CGI::Session manual. Also includes library architecture and driver specifications.
-
-=item *
-
-We also provide mailing lists for CGI::Session users. To subscribe to the list or browse the archives visit https://lists.sourceforge.net/lists/listinfo/cgi-session-user
-
-=item *
-
-B<RFC 2965> - "HTTP State Management Mechanism" found at ftp://ftp.isi.edu/in-notes/rfc2965.txt
-
-=item *
-
-L<CGI|CGI> - standard CGI library
-
-=item *
-
-L<Apache::Session|Apache::Session> - another fine alternative to CGI::Session.
-
-=back
+CGI::Session provides an easy, reliable and modular session management system across HTTP requests.
 
 =head1 METHODS
 
@@ -556,7 +519,9 @@ Following is the overview of all the available methods accessible via CGI::Sessi
 
 =head2 new( $dsn, $query||$sid, \%dsn_args )
 
-Constructor. Returns new session object, or undef on failure. Error message is accessible through L<errstr() - class method|CGI::Session::ErrorHandler/errstr>. If called on an already initialized session will re-initialize the session based on already configured object. This is only useful after a call to L<load()|/"load">.
+=head2 new( $dsn, $query||$sid, \%dsn_args, \%session_params )
+
+Constructor. Returns new session object, or undef on failure. Error message is accessible through L<errstr() - class method|CGI::Session::ErrorHandler/"errstr()">. If called on an already initialized session will re-initialize the session based on already configured object. This is only useful after a call to L<load()|/"load()">.
 
 Can accept up to three arguments, $dsn - Data Source Name, $query||$sid - query object OR a string representing session id, and finally, \%dsn_args, arguments used by $dsn components.
 
@@ -572,6 +537,14 @@ If called with two arguments first will be treated as $dsn, and second will be t
     $s = CGI::Session->new("serializer:storable;id:incr", $sid);
     # etc...
 
+Briefly, C<new()> will return an initialized session object with a valid id, whereas C<load()> may return
+an empty session object with an undefined id.
+
+Tests are provided (t/new_with_undef.t and t/load_with_undef.t) to clarify the result of calling C<new()> and C<load()>
+with undef, or with an initialized CGI object with an undefined or fake CGISESSID.
+
+You are strongly advised to run the old-fashioned 'make test TEST_FILES=t/new_with_undef.t TEST_VERBOSE=1'
+or the new-fangled 'prove -v t/new_with_undef.t', for both new*.t and load*.t, and examine the output.
 
 Following data source components are supported:
 
@@ -594,22 +567,36 @@ B<id> - ID generator to use when new session is to be created. Available ID gene
 
 For example, to get CGI::Session store its data using DB_File and serialize data using FreezeThaw:
 
-    $s = new CGI::Session("driver:DB_File;serializer:FreezeThaw", undef);
+    $s = CGI::Session->new("driver:DB_File;serializer:FreezeThaw", undef);
 
 If called with three arguments, first two will be treated as in the previous example, and third argument will be C<\%dsn_args>, which will be passed to C<$dsn> components (namely, driver, serializer and id generators) for initialization purposes. Since all the $dsn components must initialize to some default value, this third argument should not be required for most drivers to operate properly.
 
+If called with four arguments, the first three match previous examples. The fourth argument must be a hash reference with parameters to be used by the CGI::Session object. (see \%session_params above )
+
+The following is a list of the current keys:
+
+=over
+
+=item *
+
+B<name> - Name to use for the cookie/query parameter name. This defaults to CGISESSID. This can be altered or accessed by the C<name> accessor.
+
+=back
+
 undef is acceptable as a valid placeholder to any of the above arguments, which will force default behavior.
 
 =head2 load()
 
-=head2 load($query||$sid)
+=head2 load( $query||$sid )
 
-=head2 load($dsn, $query||$sid)
+=head2 load( $dsn, $query||$sid )
 
-=head2 load($dsn, $query, \%dsn_args);
+=head2 load( $dsn, $query, \%dsn_args )
+
+=head2 load( $dsn, $query, \%dsn_args, \%session_params )
 
 Accepts the same arguments as new(), and also returns a new session object, or
-undef on failure.  The difference is, L<new()|/"new"> can create new session if
+undef on failure.  The difference is, L<new()|/"new()"> can create a new session if
 it detects expired and non-existing sessions, but C<load()> does not.
 
 C<load()> is useful to detect expired or non-existing sessions without forcing the library to create new sessions. So now you can do something like this:
@@ -627,7 +614,16 @@ C<load()> is useful to detect expired or non-existing sessions without forcing t
         $s = $s->new() or die $s->errstr;
     }
 
-Notice, all I<expired> sessions are empty, but not all I<empty> sessions are expired!
+Notice: All I<expired> sessions are empty, but not all I<empty> sessions are expired!
+
+Briefly, C<new()> will return an initialized session object with a valid id, whereas C<load()> may return
+an empty session object with an undefined id.
+
+Tests are provided (t/new_with_undef.t and t/load_with_undef.t) to clarify the result of calling C<new()> and C<load()>
+with undef, or with an initialized CGI object with an undefined or fake CGISESSID.
+
+You are strongly advised to run the old-fashioned 'make test TEST_FILES=t/new_with_undef.t TEST_VERBOSE=1'
+or the new-fangled 'prove -v t/new_with_undef.t', for both new*.t and load*.t, and examine the output.
 
 =cut
 
@@ -637,7 +633,7 @@ Notice, all I<expired> sessions are empty, but not all I<empty> sessions are exp
 sub load {
     my $class = shift;
     return $class->set_error( "called as instance method")    if ref $class;
-    return $class->set_error( "Too many arguments")  if @_ > 4;
+    return $class->set_error( "Too many arguments provided to load()")  if @_ > 5;
 
     my $self = bless {
         _DATA       => {
@@ -657,11 +653,11 @@ sub load {
         _OBJECTS    => {},          # keeps necessary objects
         _DRIVER_ARGS=> {},          # arguments to be passed to driver
         _CLAIMED_ID => undef,       # id **claimed** by client
-        _STATUS     => 0,           # status of the session object
+        _STATUS     => STATUS_UNSET,# status of the session object
         _QUERY      => undef        # query object
     }, $class;
 
-    my ($dsn,$query_or_sid,$dsn_args,$update_atime);
+    my ($dsn,$query_or_sid,$dsn_args,$read_only,$params);
     # load($query||$sid)
     if ( @_ == 1 ) {
         $self->_set_query_or_sid($_[0]);
@@ -669,12 +665,24 @@ sub load {
     # Two or more args passed:
     # load($dsn, $query||$sid)
     elsif ( @_ > 1 ) {
-        ($dsn, $query_or_sid, $dsn_args,$update_atime) = @_;
+        ($dsn, $query_or_sid, $dsn_args,$read_only) = @_;
 
-        # Since $update_atime is not part of the public API
-        # we ignore any value but the one we use internally: 0.
-        if (defined $update_atime and $update_atime ne '0') {
-            return $class->set_error( "Too many arguments");
+        # Make it backwards-compatible (update_atime is an undocumented key in %$params).
+        # In fact, update_atime as a key is not used anywhere in the code as yet.
+        # This patch is part of the patch for RT#33437.
+        if ( ref $read_only and ref $read_only eq 'HASH' ) {
+            $params = {%$read_only};
+            $read_only = $params->{'read_only'};
+
+            if ($params->{'name'}) {
+                $self->{_NAME} = $params->{'name'};
+            }
+        }
+
+        # Since $read_only is not part of the public API
+        # we ignore any value but the one we use internally: 1.
+        if (defined $read_only and $read_only != '1') {
+            return $class->set_error( "Too many arguments to load(). First extra argument was: $read_only");
          }
 
         if ( defined $dsn ) {      # <-- to avoid 'Uninitialized value...' warnings
@@ -690,6 +698,9 @@ sub load {
 
     $self->_load_pluggables();
 
+    # Did load_pluggable fail? If so, return undef, just like $class->set_error() would
+    return undef if $class->errstr;
+
     if (not defined $self->{_CLAIMED_ID}) {
         my $query = $self->query();
         eval {
@@ -725,20 +736,12 @@ sub load {
         return $self->set_error( "Invalid data structure returned from thaw()" );
     }
 
-    # checking if previous session ip matches current ip
-    if($CGI::Session::IP_MATCH) {
-      unless($self->_ip_matches) {
-        $self->_set_status( STATUS_DELETED );
-        $self->flush;
-        return $self;
-      }
-    }
 
     # checking for expiration ticker
     if ( $self->{_DATA}->{_SESSION_ETIME} ) {
         if ( ($self->{_DATA}->{_SESSION_ATIME} + $self->{_DATA}->{_SESSION_ETIME}) <= time() ) {
-            $self->_set_status( STATUS_EXPIRED );   # <-- so client can detect expired sessions
-            $self->_set_status( STATUS_DELETED );   # <-- session should be removed from database
+            $self->_set_status( STATUS_EXPIRED |    # <-- so client can detect expired sessions
+                STATUS_DELETED );                   # <-- session should be removed from database
             $self->flush();                         # <-- flush() will do the actual removal!
             return $self;
         }
@@ -746,16 +749,27 @@ sub load {
 
     # checking expiration tickers of individuals parameters, if any:
     my @expired_params = ();
-    while (my ($param, $max_exp_interval) = each %{ $self->{_DATA}->{_SESSION_EXPIRE_LIST} } ) {
-        if ( ($self->{_DATA}->{_SESSION_ATIME} + $max_exp_interval) <= time() ) {
-            push @expired_params, $param;
+    if ( $self->{_DATA}->{_SESSION_EXPIRE_LIST} ) {
+        while (my ($param, $max_exp_interval) = each %{ $self->{_DATA}->{_SESSION_EXPIRE_LIST} } ) {
+            if ( ($self->{_DATA}->{_SESSION_ATIME} + $max_exp_interval) <= time() ) {
+                push @expired_params, $param;
+            }
         }
     }
     $self->clear(\@expired_params) if @expired_params;
 
-    # We update the atime by default, but if this (otherwise undocoumented)
-    # parameter is explicitly set to false, we'll turn the behavior off
-    if ( ! defined $update_atime ) {
+
+
+    if (not defined $read_only) {
+        # checking if previous session ip matches current ip
+        if($CGI::Session::IP_MATCH) {
+            unless($self->ip_matches) {
+                $self->_set_status( STATUS_DELETED );
+                $self->flush;
+                return $self;
+            }
+        }
+
         $self->{_DATA}->{_SESSION_ATIME} = time();      # <-- updating access time
         $self->_set_status( STATUS_MODIFIED );          # <-- access time modified above
     }
@@ -787,7 +801,7 @@ sub _load_pluggables {
                        id         => "ID",
                        );
     my $dsn = $self->{_DSN};
-    foreach my $plug qw(driver serializer id) {
+    for my $plug (qw(driver serializer id)) {
         my $mod_name = $dsn->{ $plug };
         if (not defined $mod_name) {
             $mod_name = $DEFAULT_FOR{ $plug };
@@ -841,7 +855,7 @@ a warning and undef will be returned.
 
 =head2 param_hashref()
 
-B<Deprecated>. Use L<dataref()|/"dataref"> instead.
+B<Deprecated>. Use L<dataref()|/"dataref()"> instead.
 
 =head2 dataref()
 
@@ -860,7 +874,7 @@ Useful for having all session data in a hashref, but too risky to update.
 
 =head2 save_param($query, \@list)
 
-Saves query parameters to session object. In other words, it's the same as calling L<param($name, $value)|/"param"> for every single query parameter returned by C<< $query->param() >>. The first argument, if present, should be either CGI object or any object which can provide param() method. If it's undef, defaults to the return value of L<query()|/"query">, which returns C<< CGI->new >>. If second argument is present and is a reference to an array, only those query parameters found in the array will be stored in the session. undef is a valid placeholder for any argument to force default behavior.
+Saves query parameters to session object. In other words, it's the same as calling L<param($name, $value)|/"param($name)"> for every single query parameter returned by C<< $query->param() >>. The first argument, if present, should be either CGI object or any object which can provide param() method. If it's undef, defaults to the return value of L<query()|/"query()">, which returns C<< CGI->new >>. If second argument is present and is a reference to an array, only those query parameters found in the array will be stored in the session. undef is a valid placeholder for any argument to force default behavior.
 
 =head2 load_param()
 
@@ -883,18 +897,20 @@ reference to an array, only the named parameters are cleared.
 
 =head2 flush()
 
-Synchronizes data in memory  with the copy serialized by the driver. Call flush() 
+Synchronizes data in memory with the copy serialized by the driver. Call flush()
 if you need to access the session from outside the current session object. You should
-at least call flush() before your program exits. 
+call flush() sometime before your program exits.
 
 As a last resort, CGI::Session will automatically call flush for you just
-before the program terminates or session object goes out of scope. This automatic
-behavior was the recommended behavior until the 4.x series. Automatic flushing
-has since proven to be unreliable, and in some cases is now required in places
-that worked with 3.x. For further details see:
+before the program terminates or session object goes out of scope. Automatic
+flushing has proven to be unreliable, and in some cases is now required
+in places that worked with CGI::Session 3.x.
 
- http://rt.cpan.org/Ticket/Display.html?id=17541
- http://rt.cpan.org/Ticket/Display.html?id=17299
+Always explicitly calling C<flush()> on the session before the
+program exits is recommended. For extra safety, call it immediately after
+every important session update.
+
+Also see L<A Warning about Auto-flushing>
 
 =head2 atime()
 
@@ -911,7 +927,7 @@ Read-only method. Returns the time when the session was first created in seconds
 
 =head2 expire($param, $time)
 
-Sets expiration interval relative to L<atime()|/"atime">.
+Sets expiration interval relative to L<atime()|/"atime()">.
 
 If used with no arguments, returns the expiration interval if it was ever set. If no expiration was ever set, returns undef. For backwards compatibility, a method named C<etime()> does the same thing.
 
@@ -1055,9 +1071,24 @@ Actually, the above code is nothing but waste. The same effect could've been ach
 
 L<is_empty()|/"is_empty"> is useful only if you wanted to catch requests for expired sessions, and create new session afterwards. See L<is_expired()|/"is_expired"> for an example.
 
+=head2 ip_match()
+
+Returns true if $ENV{REMOTE_ADDR} matches the remote address stored in the session.
+
+If you have an application where you are sure your users' IPs are constant
+during a session, you can consider enabling an option to make this check:
+
+    use CGI::Session '-ip_match';
+
+Usually you don't call ip_match() directly, but by using the above method. It is useful
+only if you want to call it inside of coderef passed to the L<find()|/"find( \&code )"> method. 
+
 =head2 delete()
 
-Deletes a session from the data store and empties session data from memory, completely, so subsequent read/write requests on the same object will fail. Technically speaking, it will only set object's status to I<STATUS_DELETED> and will trigger L<flush()|/"flush">, and flush() will do the actual removal.
+Sets the objects status to be "deleted".  Subsequent read/write requests on the
+same object will fail.  To physically delete it from the data store you need to call L<flush()|/"flush()">.
+CGI::Session attempts to do this automatically when the object is being destroyed (usually as
+the script exits), but see L<A Warning about Auto-flushing>.
 
 =head2 find( \&code )
 
@@ -1071,14 +1102,15 @@ The following line, for instance, will remove sessions already expired, but whic
 
     CGI::Session->find( sub {} );
 
-Notice, above \&code didn't have to do anything, because load(), which is called to initialize sessions inside find(), will automatically remove expired sessions. Following example will remove all the objects that are 10+ days old:
+Notice, above \&code didn't have to do anything, because load(), which is called to initialize sessions inside L<find()|/"find( \&code )">, will automatically remove expired sessions. Following example will remove all the objects that are 10+ days old:
 
     CGI::Session->find( \&purge );
     sub purge {
         my ($session) = @_;
         next if $session->is_empty;    # <-- already expired?!
         if ( ($session->ctime + 3600*240) <= time() ) {
-            $session->delete() or warn "couldn't remove " . $session->id . ": " . $session->errstr;
+            $session->delete();
+            $session->flush(); # Recommended practice says use flush() after delete().
         }
     }
 
@@ -1151,6 +1183,17 @@ The default value of this hashref is undef.
 
 B<Note:> find() is meant to be convenient, not necessarily efficient. It's best suited in cron scripts.
 
+=head2 name($new_name)
+
+The $new_name parameter is optional. If supplied it sets the query or cookie parameter name to be used.
+
+It defaults to I<$CGI::Session::NAME>, which defaults to I<CGISESSID>.
+
+You are strongly discouraged from using the global variable I<$CGI::Session::NAME>, since it is
+deprecated (as are all global variables) and will be removed in a future version of this module.
+
+Return value: The current query or cookie parameter name.
+
 =head1 MISCELLANEOUS METHODS
 
 =head2 remote_addr()
@@ -1173,25 +1216,27 @@ Returns a dump of the session object. Useful for debugging purposes only.
 
 =head2 header()
 
-Replacement for L<CGI.pm|CGI>'s header() method. Without this method, you usually need to create a CGI::Cookie object and send it as part of the HTTP header:
+A wrapper for C<CGI>'s header() method. Calling this method
+is equivalent to something like this:
 
     $cookie = CGI::Cookie->new(-name=>$session->name, -value=>$session->id);
-    print $cgi->header(-cookie=>$cookie);
+    print $cgi->header(-cookie=>$cookie, @_);
 
 You can minimize the above into:
 
     print $session->header();
 
-It will retrieve the name of the session cookie from C<$session->name()> which defaults to C<$CGI::Session::NAME>. If you want to use a different name for your session cookie, do something like following before creating session object:
+It will retrieve the name of the session cookie from C<$session->name()> which defaults to C<$CGI::Session::NAME>. If you want to use a different name for your session cookie, do something like this before creating session object:
 
     CGI::Session->name("MY_SID");
-    $session = new CGI::Session(undef, $cgi, \%attrs);
+    $session = CGI::Session->new(undef, $cgi, \%attrs);
 
-Now, $session->header() uses "MY_SID" as a name for the session cookie.
+Now, $session->header() uses "MY_SID" as the name for the session cookie. For all additional options that can
+be passed, see the C<header()> docs in C<CGI>.
 
 =head2 query()
 
-Returns query object associated with current session object. Default query object class is L<CGI.pm|CGI>.
+Returns query object associated with current session object. Default query object class is C<CGI>.
 
 =head2 DEPRECATED METHODS
 
@@ -1208,7 +1253,7 @@ CGI::Session consists of several components such as L<drivers|"DRIVERS">, L<seri
 
 =head2 DRIVERS
 
-Following drivers are included in the standard distribution:
+The following drivers are included in the standard distribution:
 
 =over 4
 
@@ -1233,6 +1278,8 @@ Full name: B<CGI::Session::Driver::sqlite>
 
 =back
 
+Other drivers are available from CPAN.
+
 =head2 SERIALIZERS
 
 =over 4
@@ -1257,16 +1304,11 @@ Full name: B<CGI::Session::Serialize::freezethaw>
 L<yaml|CGI::Session::Serialize::yaml> - serializes data using YAML. Requires L<YAML> or L<YAML::Syck>.
 Full name: B<CGI::Session::Serialize::yaml>
 
-=item *
-
-L<json|CGI::Session::Serialize::json> - serializes data using JSON. Requires L<JSON::Syck>.
-Full name: B<CGI::Session::Serialize::json>
-
 =back
 
 =head2 ID GENERATORS
 
-Following ID generators are available:
+The following ID generators are included in the standard distribution.
 
 =over 4
 
@@ -1285,6 +1327,105 @@ L<static|CGI::Session::ID::static> - generates static session ids. B<CGI::Sessio
 
 =back
 
+=head1 A Warning about Auto-flushing
+
+Auto-flushing can be unreliable for the following reasons. Explicit flushing
+after key session updates is recommended. 
+
+=over 4
+
+=item If the C<DBI> handle goes out of scope before the session variable
+
+For database-stored sessions, if the C<DBI> handle has gone out of scope before
+the auto-flushing happens, auto-flushing will fail.
+
+=item Circular references
+
+If the calling code contains a circular reference, it's possible that your
+C<CGI::Session> object will not be destroyed until it is too late for
+auto-flushing to work. You can find circular references with a tool like
+L<Devel::Cycle>.
+
+In particular, these modules are known to contain circular references which
+lead to this problem:
+
+=over 4
+
+=item CGI::Application::Plugin::DebugScreen V 0.06
+
+=item CGI::Application::Plugin::ErrorPage before version 1.20
+
+=back
+
+=item Signal handlers
+
+If your application may receive signals, there is an increased chance that the
+signal will arrive after the session was updated but before it is auto-flushed
+at object destruction time.
+
+=back
+
+=head1 A Warning about UTF8
+
+You are strongly encouraged to refer to, at least, the first of these articles, for help with UTF8.
+
+L<http://en.wikibooks.org/wiki/Perl_Programming/Unicode_UTF-8>
+
+L<http://perl.bristolbath.org/blog/lyle/2008/12/giving-cgiapplication-internationalization-i18n.html>
+
+L<http://metsankulma.homelinux.net/cgi-bin/l10n_example_4/main.cgi>
+
+L<http://rassie.org/archives/247>
+
+L<http://www.di-mgt.com.au/cryptoInternational2.html>
+
+Briefly, these are the issues:
+
+=over 4
+
+=item The file containing the source code of your program
+
+Consider "use utf8;" or "use encoding 'utf8';".
+
+=item Influencing the encoding of the program's input
+
+Use:
+
+    binmode STDIN, ":encoding(utf8)";.
+
+    Of course, the program can get input from other sources, e.g. HTML template files, not just STDIN.
+
+=item Influencing the encoding of the program's output
+
+Use:
+
+    binmode STDOUT, ":encoding(utf8)";
+
+    When using CGI.pm, you can use $q->charset('UTF-8'). This is the same as passing 'UTF-8' to CGI's C<header()> method.
+
+    Alternately, when using CGI::Session, you can use $session->header(charset => 'utf-8'), which will be
+    passed to the query object's C<header()> method. Clearly this is preferable when the query object might not be
+    of type CGI.
+
+    See L</header()> for a fuller discussion of the use of the C<header()> method in conjunction with cookies.
+
+=back
+
+=head1 TRANSLATIONS
+
+This document is also available in Japanese.
+
+=over 4
+
+=item o
+
+Translation based on 4.14: http://digit.que.ne.jp/work/index.cgi?Perldoc/ja
+
+=item o
+
+Translation based on 3.11, including Cookbook and Tutorial: http://perldoc.jp/docs/modules/CGI-Session-3.11/
+
+=back
 
 =head1 CREDITS
 
@@ -1292,7 +1433,7 @@ CGI::Session evolved to what it is today with the help of following developers.
 
 =over 4
 
-=item Andy Lester 
+=item Andy Lester
 
 =item Brian King E<lt>mrbbking@mac.comE<gt>
 
@@ -1302,14 +1443,23 @@ CGI::Session evolved to what it is today with the help of following developers.
 
 =item Igor Plisco E<lt>igor@plisco.ruE<gt>
 
-=item Mark Stosberg 
+=item Mark Stosberg
 
 =item Matt LeBlanc E<lt>mleblanc@cpan.orgE<gt>
 
 =item Shawn Sorichetti
 
+=item Ron Savage
+
+=item Rhesa Rozendaal
+
+He suggested Devel::Cycle to help debugging.
+
 =back
 
+Also, many people on the CGI::Application and CGI::Session mailing lists have contributed ideas and
+suggestions, and battled publicly with bugs, all of which has helped.
+
 =head1 COPYRIGHT
 
 Copyright (C) 2001-2005 Sherzod Ruzmetov E<lt>sherzodr@cpan.orgE<gt>. All rights reserved.
@@ -1318,18 +1468,18 @@ This library is free software. You can modify and or distribute it under the sam
 =head1 PUBLIC CODE REPOSITORY
 
 You can see what the developers have been up to since the last release by
-checking out the code repository. You can browse the Subversion repository from here:
+checking out the code repository. You can browse the git repository from here:
 
- http://svn.cromedome.net/
+ http://github.com/cromedome/cgi-session/tree/master
 
-Or check it directly with C<svn> from here:
+Or check out the code with:
 
- svn://svn.cromedome.net/CGI-Session
+ git clone git://github.com/cromedome/cgi-session.git
 
 =head1 SUPPORT
 
-If you need help using CGI::Session consider the mailing list. You can ask the list by sending your questions to
-cgi-session-user@lists.sourceforge.net .
+If you need help using CGI::Session, ask on the mailing list. You can ask the
+list by sending your questions to cgi-session-user@lists.sourceforge.net .
 
 You can subscribe to the mailing list at https://lists.sourceforge.net/lists/listinfo/cgi-session-user .
 
@@ -1337,29 +1487,39 @@ Bug reports can be submitted at http://rt.cpan.org/NoAuth/ReportBug.html?Queue=C
 
 =head1 AUTHOR
 
-Sherzod Ruzmetov E<lt>sherzodr@cpan.orgE<gt>, http://author.handalak.com/
+Sherzod Ruzmetov C<sherzodr@cpan.org>
 
 Mark Stosberg became a co-maintainer during the development of 4.0. C<markstos@cpan.org>.
 
-=head1 SEE ALSO
+Ron Savage became a co-maintainer during the development of 4.30. C<rsavage@cpan.org>.
+
+If you would like support, ask on the mailing list as describe above. The
+maintainers and other users are subscribed to it. 
+
+=head1 SEE ALSO 
+
+To learn more both about the philosophy and CGI::Session programming style,
+consider the following:
 
 =over 4
 
 =item *
 
-L<CGI::Session::Tutorial|CGI::Session::Tutorial> - extended CGI::Session manual
+L<CGI::Session::Tutorial|CGI::Session::Tutorial> - extended CGI::Session manual. Also includes library architecture and driver specifications.
 
 =item *
 
-B<RFC 2965> - "HTTP State Management Mechanism" found at ftp://ftp.isi.edu/in-notes/rfc2965.txt
+We also provide mailing lists for CGI::Session users. To subscribe to the list
+or browse the archives visit
+https://lists.sourceforge.net/lists/listinfo/cgi-session-user
+
+=item * B<RFC 2109> - The primary spec for cookie handing in use, defining the  "Cookie:" and "Set-Cookie:" HTTP headers.
+Available at L<http://www.ietf.org/rfc/rfc2109.txt>. A newer spec, RFC 2965 is meant to obsolete it with "Set-Cookie2" 
+and "Cookie2" headers, but even of 2008, the newer spec is not widely supported. See L<http://www.ietf.org/rfc/rfc2965.txt>
 
 =item *
 
-L<CGI|CGI> - standard CGI library
-
-=item *
-
-L<Apache::Session|Apache::Session> - another fine alternative to CGI::Session
+L<Apache::Session|Apache::Session> - an alternative to CGI::Session.
 
 =back
 
