@@ -1,17 +1,17 @@
 package CGI::Session::ID::md5;
 
-# $Id: md5.pm 351 2006-11-24 14:16:50Z markstos $
+# $Id$
 
 use strict;
 use Digest::MD5;
 use CGI::Session::ErrorHandler;
 
-$CGI::Session::ID::md5::VERSION = '4.20';
+$CGI::Session::ID::md5::VERSION = '4.43';
 @CGI::Session::ID::md5::ISA     = qw( CGI::Session::ErrorHandler );
 
 *generate = \&generate_id;
 sub generate_id {
-    my $md5 = new Digest::MD5();
+    my $md5 = Digest::MD5->new();
     $md5->add($$ , time() , rand(time) );
     return $md5->hexdigest();
 }
@@ -28,7 +28,7 @@ CGI::Session::ID::md5 - default CGI::Session ID generator
 =head1 SYNOPSIS
 
     use CGI::Session;
-    $s = new CGI::Session("id:md5", undef);
+    $s = CGI::Session->new("id:md5", undef);
 
 =head1 DESCRIPTION
 
